@@ -5,21 +5,47 @@
 Utilities for sending files over ssh using the scp1 protocol.
 """
 
-__version__ = '0.13.6'
+__version__ = '0.14.5'
 
 import locale
 import os
 import re
 from socket import timeout as SocketTimeout
-import types
+
+
+SCP_COMMAND = b'scp'
+
+PATH_TYPES = (str, bytes)
+
+try:
+    import pathlib
+except ImportError:
+    pathlib = None
+else:
+    PATH_TYPES += pathlib.PurePath,
+try:
+    PATH_TYPES += unicode,
+except NameError:
+    pass
+
+try:
+    from typing import IO, TYPE_CHECKING, AnyStr, Callable, Iterable, Optional, Tuple, Union
+
+    if TYPE_CHECKING:
+        import paramiko.transport
+
+    # this is some magic to make sure pyright doesn't get too confused with pathlib potentially being a nullable variable
+    import pathlib
+    PurePath = pathlib.PurePath
+
+    PathTypes = Union[str, bytes, PurePath]
+except ImportError:
+    pass
 
 
 # this is quote from the shlex module, added in py3.3
 _find_unsafe = re.compile(br'[^\w@%+=:,./~-]').search
 
-SCP_COMMAND = b'scp'
-
-
 def _sh_quote(s):
     """Return a shell-escaped version of the string `s`."""
     if not s:
@@ -41,6 +67,8 @@ def asbytes(s):
     """
     if isinstance(s, bytes):
         return s
+    elif pathlib and isinstance(s, pathlib.PurePath):
+        return bytes(s)
     else:
         return s.encode('utf-8')
 
@@ -88,6 +116,7 @@ class SCPClient(object):
     """
     def __init__(self, transport, buff_size=16384, socket_timeout=10.0,
                  progress=None, progress4=None, sanitize=_sh_quote):
+        # type: (paramiko.transport.Transport, int, float, Optional[Callable[[bytes, int, int], None]], Optional[Callable[[bytes, int, int, Tuple[str, int]], None]], Callable[[bytes], bytes]) -> None
         """
         Create an scp1 client.
 
@@ -102,7 +131,8 @@ class SCPClient(object):
         @param progress4: callback - called with (filename, size, sent, peername)
             during transfers. peername is a tuple contains (IP, PORT)
         @param sanitize: function - called with filename, should return
-            safe or escaped string.  Uses _sh_quote by default.
+            safe or escaped string. Uses _sh_quote by default. Set to ``False``
+            to disable.
         @type progress: function(string, int, int, tuple)
         """
         self.transport = transport
@@ -122,7 +152,10 @@ class SCPClient(object):
         self._depth = 0
         self._rename = False
         self._utime = None
-        self.sanitize = sanitize
+        if sanitize is False:
+            self.sanitize = lambda x: x
+        else:
+            self.sanitize = sanitize
         self._dirtimes = {}
         self.peername = self.transport.getpeername()
         self.scp_command = SCP_COMMAND
@@ -136,6 +169,7 @@ class SCPClient(object):
 
     def put(self, files, remote_path=b'.',
             recursive=False, preserve_times=False):
+        # type: (Union[PathTypes, Iterable[PathTypes]], Union[str, bytes], bool, bool) -> None
         """
         Transfer files and directories to remote host.
 
@@ -160,8 +194,10 @@ class SCPClient(object):
                                   self.sanitize(asbytes(remote_path)))
         self._recv_confirm()
 
-        if not isinstance(files, (list, tuple)):
+        if isinstance(files, PATH_TYPES):
             files = [files]
+        else:
+            files = list(files)
 
         if recursive:
             self._send_recursive(files)
@@ -171,6 +207,7 @@ class SCPClient(object):
         self.close()
 
     def putfo(self, fl, remote_path, mode='0644', size=None):
+        # type: (IO[AnyStr], Union[str, bytes], Union[str, bytes], Optional[int]) -> None
         """
         Transfer file-like object to remote host.
 
@@ -199,12 +236,13 @@ class SCPClient(object):
 
     def get(self, remote_path, local_path='',
             recursive=False, preserve_times=False):
+        # type: (PathTypes, Union[str, bytes], bool, bool) -> None
         """
         Transfer files and directories from remote host to localhost.
 
-        @param remote_path: path to retrieve from remote host. since this is
-            evaluated by scp on the remote host, shell wildcards and
-            environment variables may be used.
+        @param remote_path: path to retrieve from remote host. Note that
+            wildcards will be escaped unless you changed the `sanitize`
+            function.
         @type remote_path: str
         @param local_path: path in which to receive files locally
         @type local_path: str
@@ -214,8 +252,10 @@ class SCPClient(object):
             and directories.
         @type preserve_times: bool
         """
-        if not isinstance(remote_path, (list, tuple)):
+        if isinstance(remote_path, PATH_TYPES):
             remote_path = [remote_path]
+        else:
+            remote_path = list(remote_path)
         remote_path = [self.sanitize(asbytes(r)) for r in remote_path]
         self._recv_dir = local_path or os.getcwd()
         self._depth = 0
@@ -271,8 +311,10 @@ class SCPClient(object):
             if self.preserve_times:
                 self._send_time(mtime, atime)
             fl = open(name, 'rb')
-            self._send_file(fl, name, mode, size)
-            fl.close()
+            try:
+                self._send_file(fl, name, mode, size)
+            finally:
+                fl.close()
 
     def _send_file(self, fl, name, mode, size):
         basename = asbytes(os.path.basename(name))
@@ -320,6 +362,7 @@ class SCPClient(object):
 
     def _send_recursive(self, files):
         for base in files:
+            base = asbytes(base)
             if not os.path.isdir(base):
                 # filename mixed into the bunch
                 self._send_files([base])
@@ -383,7 +426,7 @@ class SCPClient(object):
             # wait for command as long as we're open
             self.channel.sendall('\x00')
             msg = self.channel.recv(1024)
-            if not msg:  # chan closed while recving
+            if not msg:  # chan closed while receiving
                 break
             assert msg[-1:] == b'\n'
             msg = msg[:-1]
@@ -459,16 +502,15 @@ class SCPClient(object):
             msg = chan.recv(512)
             if msg and msg[0:1] != b'\x00':
                 raise SCPException(asunicode(msg[1:]))
-        except SocketTimeout:
-            chan.close()
-            raise SCPException('Error receiving, socket.timeout')
 
-        file_hdl.truncate()
-        try:
+            file_hdl.truncate()
             os.utime(path, self._utime)
             self._utime = None
             os.chmod(path, mode)
             # should we notify the other end?
+        except SocketTimeout:
+            chan.close()
+            raise SCPException('Error receiving, socket.timeout')
         finally:
             file_hdl.close()
         # '\x00' confirmation sent in _recv_all
@@ -527,6 +569,7 @@ class SCPException(Exception):
 
 def put(transport, files, remote_path=b'.',
         recursive=False, preserve_times=False):
+    # type: (paramiko.transport.Transport, Union[PathTypes, Iterable[PathTypes]], Union[str, bytes], bool, bool) -> None
     """
     Transfer files and directories to remote host.
 
@@ -551,6 +594,7 @@ def put(transport, files, remote_path=b'.',
 
 def get(transport, remote_path, local_path='',
         recursive=False, preserve_times=False):
+    # type: (paramiko.transport.Transport, PathTypes, Union[str, bytes], bool, bool) -> None
     """
     Transfer files and directories from remote host to localhost.
 
@@ -559,9 +603,8 @@ def get(transport, remote_path, local_path='',
 
     @param transport: an paramiko L{Transport}
     @type transport: L{Transport}
-    @param remote_path: path to retrieve from remote host. since this is
-        evaluated by scp on the remote host, shell wildcards and environment
-        variables may be used.
+    @param remote_path: path to retrieve from remote host. Note that wildcards
+        will be escaped unless you changed the `sanitize` function.
     @type remote_path: str
     @param local_path: path in which to receive files locally
     @type local_path: str
